// ----------------------------------------------------------------------------
// ...\StringGridPopUpMenu\Main.pas
//
// Purpose:
//   Main form. Example of Delphi app with string grid and context menu.
//   Context menu can be opened with right-click on mouse or by apps key on
//   keyboard.
//
// Created:
//   2018-08-11  Foersom
//
// License: CC-BY 4.0
// ----------------------------------------------------------------------------

unit Main;

interface // ------------------------------------------------------------------

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.Grids, Vcl.Menus, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TMainForm = class(TForm)
    StringGrid1: TStringGrid;
    GridPopupMenu: TPopupMenu;
    OptionA: TMenuItem;
    OptionB: TMenuItem;
    OptionC: TMenuItem;
    OutputMemo: TMemo;
    ExitButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure OptionAClick(Sender: TObject);
    procedure OptionBClick(Sender: TObject);
    procedure OptionCClick(Sender: TObject);
    procedure StringGrid1ContextPopup(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
    procedure ExitButtonClick(Sender: TObject);
  private
    { Private declarations }
    procedure InitGrid;
    procedure OutputMemoWriteln(s: string);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation // -------------------------------------------------------------

{$R *.dfm}

procedure TMainForm.OutputMemoWriteln(s: string);
begin
  OutputMemo.Lines.Add(s);
end;

procedure TMainForm.OptionAClick(Sender: TObject);
begin
  with StringGrid1 do
    OutputMemoWriteln(format('Option A; Col,Row: %d,%d; Cell: %s',[Col,Row,Cells[Col,Row]]));
end;

procedure TMainForm.OptionBClick(Sender: TObject);
begin
  with StringGrid1 do
    OutputMemoWriteln(format('Option B; Col,Row: %d,%d; Cell: %s',[Col,Row,Cells[Col,Row]]));
end;

procedure TMainForm.OptionCClick(Sender: TObject);
begin
  with StringGrid1 do
    OutputMemoWriteln(format('Option C; Col,Row: %d,%d; Cell: %s',[Col,Row,Cells[Col,Row]]));
end;

procedure TMainForm.StringGrid1ContextPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
var
  C,R: integer;
  Rect: TRect;
begin
  OutputMemoWriteln('-- TMainForm.StringGrid1ContextPopup()');
  OutputMemoWriteln(format('MousePos X,Y: %d,%d',[MousePos.X,MousePos.Y]));

  if MousePos=Point(-1,-1) then
  begin  // VK_Apps key
    // get current cell's rectangle
    Rect:=StringGrid1.CellRect(StringGrid1.Col, StringGrid1.Row);
    MousePos:=Rect.BottomRight;
    // place menu inside lower right corner of cell
    dec(MousePos.X,8);
    dec(MousePos.Y,8);
    OutputMemoWriteln(format('VK_Apps key, form MousePos x,y: %d,%d',[MousePos.X,MousePos.Y]));
  end
  else
  begin  // mouse context click
    StringGrid1.MouseToCell(MousePos.X,MousePos.Y,C,R);
        // returns C = -1 and R = -1 if mouse outside grid
    if (C>=1) and (R>=1) then
    begin  // set selected cell to mouse cursor pos, but fixed col and row excluded
      StringGrid1.Col:=C;
      StringGRid1.Row:=R;
    end;
    OutputMemoWriteln(format('Mouse, form MousePos x,y: %d,%d',[MousePos.X,MousePos.Y]));
  end;

  // convert to screen coordinates
  MousePos:=StringGrid1.ClientToScreen(MousePos);
  GridPopupMenu.Popup(MousePos.X,MousePos.Y);
  Handled:=true;
end;

procedure TMainForm.InitGrid;
var
  R,C: integer;
begin
  with StringGrid1 do
  begin
    for R := 0 to RowCount-1 do
      for C := 0 to ColCount-1 do
        cells[C,R]:=format('C%dR%d',[C,R]);
  end;
end;

procedure TMainForm.ExitButtonClick(Sender: TObject);
begin
  Close;  // close MainForm to exit app
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  OutputMemo.Clear;
  InitGrid;
end;

end.

